# pylint: disable=missing-docstring,redefined-outer-name
import os
import subprocess
from collections.abc import Mapping, MutableMapping
from unittest import mock

from unittest_fixtures import FixtureContext, Fixtures, fixture


@fixture()
def environ(
    _fixtures: Fixtures, *, environ: Mapping[str, str] | None = None, clear: bool = True
) -> FixtureContext[MutableMapping[str, str]]:
    with mock.patch.dict(os.environ, clear=clear):
        os.environ.update(environ or {})
        yield os.environ


@fixture()
def popen(_fixtures: Fixtures) -> FixtureContext[mock.Mock]:
    with mock.patch.object(subprocess, "Popen") as mock_popen:
        yield mock_popen



"""Handler(s) for gbp-webhook-playsound"""

import subprocess as sp
from typing import Any

from gbp_webhook_playsound import get_sound_file, get_sound_player


def build_pulled(_event: Any) -> None:
    """build_pulled event handler"""
    with sp.Popen([*get_sound_player(), get_sound_file("build_pulled")]):
        pass



# pylint: disable=missing-docstring
from collections.abc import MutableMapping
from unittest import TestCase

from unittest_fixtures import Fixtures, given

from gbp_webhook_playsound import DEFAULT_SOUND
from gbp_webhook_playsound.handlers import build_pulled

from . import lib


@given(lib.environ, lib.popen)
class BuildPulledTests(TestCase):
    def test(self, fixtures: Fixtures) -> None:
        build_pulled(None)

        popen = fixtures.popen
        popen.assert_called_once_with(["pw-play", DEFAULT_SOUND])

    def test_custom_player(self, fixtures: Fixtures) -> None:
        environ: MutableMapping[str, str] = fixtures.environ
        environ["GBP_WEBHOOK_PLAYSOUND_PLAYER"] = "mpg123 -q"

        build_pulled(None)

        popen = fixtures.popen
        popen.assert_called_once_with(["mpg123", "-q", DEFAULT_SOUND])



# pylint: disable=missing-docstring,unused-argument
from collections.abc import MutableMapping
from unittest import TestCase

from unittest_fixtures import Fixtures, given

from gbp_webhook_playsound import DEFAULT_SOUND, get_sound_file

from . import lib


@given(lib.environ)
class GetSoundFileTests(TestCase):
    def test_default(self, fixtures: Fixtures) -> None:
        self.assertEqual(DEFAULT_SOUND, get_sound_file("build_pulled"))

    def test_environment_variable(self, fixtures: Fixtures) -> None:
        environ: MutableMapping[str, str] = fixtures.environ
        environ["GBP_WEBHOOK_PLAYSOUND_BUILD_PULLED"] = "/foo/bar.mp3"

        self.assertEqual("/foo/bar.mp3", get_sound_file("build_pulled"))



"""gbp-webhook-playsound"""

import importlib.resources
import os

with importlib.resources.path("gbp_webhook_playsound", "level-up-191997.mp3") as _p:
    DEFAULT_SOUND = str(_p)

del _p


def get_sound_file(event_name: str) -> str:
    """Return the path of the given event's sound file

    event_name such as "build_pulled"

    First looks for the environment variable GBP_WEBHOOK_PLAYSOUND_<event_name>, where
    event_name is capitalized. If the environment variable exists and value is not empty
    the value is returned.  Otherwise the "default" sound's path is returned.
    """
    var = f"GBP_WEBHOOK_PLAYSOUND_{event_name.upper()}"
    environ = os.environ

    if path := environ.get(var, ""):
        return path
    return DEFAULT_SOUND


def get_sound_player() -> list[str]:
    """Return the sound player executable"""
    return os.environ.get("GBP_WEBHOOK_PLAYSOUND_PLAYER", "pw-play").split()
